__author__ = 'Guillermo Avendano-Franco'


def test_good():
    """
    Simple test import pychemia                                  :
    """
    from pychemia import pcm_log
    pcm_log.debug('DEBUGGING')
    pass
